@@ -23,12 +23,6 @@
 #include "gnutls_errors.h"
 #include "gnutls_cert.h"
 
-/* include all the kx handler's definitions */
-#include "auth_anon.h"
-/* #include "auth_dhe_dss.h" */
-#include "auth_srp.h"
-#include "auth_x509.h"
-
 #define MAX_CIPHER 256
 #define MAX_MAC 256
 #define MAX_KX 256
@@ -187,6 +181,10 @@ struct gnutls_kx_algo_entry {
 };
 typedef struct gnutls_kx_algo_entry gnutls_kx_algo_entry;
 
+extern MOD_AUTH_STRUCT rsa_auth_struct;
+extern MOD_AUTH_STRUCT anon_auth_struct;
+extern MOD_AUTH_STRUCT srp_auth_struct;
+
 static const gnutls_kx_algo_entry kx_algorithms[] = {
 	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_ANON, &anon_auth_struct),
 	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_RSA, &rsa_auth_struct),
@@ -787,12 +785,6 @@ int _gnutls_version_get_major(GNUTLS_Version version)
 }
 
 /* Version Functions */
-int _gnutls_version_cmp(GNUTLS_Version ver1, GNUTLS_Version ver2)
-{
-	if (ver1 != ver2)
-		return 1;
-	return 0;
-}
 
 int
 _gnutls_version_is_supported(GNUTLS_STATE state,
@@ -1051,7 +1043,13 @@ _gnutls_supported_ciphersuites_sorted(GNUTLS_STATE state,
 	}
 
 	tmp_ciphers = gnutls_malloc(count * sizeof(GNUTLS_CipherSuite));
+	if (tmp_ciphers==NULL) return GNUTLS_E_MEMORY_ERROR;
+	
 	*ciphers = gnutls_malloc(count * sizeof(GNUTLS_CipherSuite));
+	if (*ciphers==NULL) {
+		gnutls_free(tmp_ciphers);
+		return GNUTLS_E_MEMORY_ERROR;
+	}
 
 
 	for (i = 0; i < count; i++) {
@@ -1122,22 +1120,31 @@ _gnutls_supported_ciphersuites_sorted(GNUTLS_STATE state,
 
 int
 _gnutls_supported_ciphersuites(GNUTLS_STATE state,
-			       GNUTLS_CipherSuite ** ciphers)
+			       GNUTLS_CipherSuite ** _ciphers)
 {
 
 	int i, ret_count, j;
 	int count = _gnutls_cipher_suite_count();
 	GNUTLS_CipherSuite *tmp_ciphers;
+	GNUTLS_CipherSuite* ciphers;
+
+	*_ciphers = NULL;
 
 	if (count == 0) {
-		*ciphers = NULL;
 		return 0;
 	}
 
 	tmp_ciphers = gnutls_malloc(count * sizeof(GNUTLS_CipherSuite));
-	*ciphers = gnutls_malloc(count * sizeof(GNUTLS_CipherSuite));
+	if ( tmp_ciphers==NULL)
+		return GNUTLS_E_MEMORY_ERROR;
 
+	ciphers = gnutls_malloc(count * sizeof(GNUTLS_CipherSuite));
 
+	if ( ciphers==NULL) {
+		gnutls_free( tmp_ciphers);
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+	
 	for (i = 0; i < count; i++) {
 		tmp_ciphers[i].CipherSuite[0] =
 		    cs_algorithms[i].id.CipherSuite[0];
@@ -1161,26 +1168,25 @@ _gnutls_supported_ciphersuites(GNUTLS_STATE state,
 		    < 0)
 			continue;
 
-		(*ciphers)[j].CipherSuite[0] =
-		    tmp_ciphers[i].CipherSuite[0];
-		(*ciphers)[j].CipherSuite[1] =
-		    tmp_ciphers[i].CipherSuite[1];
+		ciphers[j].CipherSuite[0] = tmp_ciphers[i].CipherSuite[0];
+		ciphers[j].CipherSuite[1] = tmp_ciphers[i].CipherSuite[1];
 		j++;
 	}
 
 	ret_count = j;
 
 	if (ret_count > 0 && ret_count != count) {
-		*ciphers =
-		    gnutls_realloc(*ciphers,
+		ciphers =
+		    gnutls_realloc(ciphers,
 				   ret_count * sizeof(GNUTLS_CipherSuite));
 	} else {
 		if (ret_count != count) {
-			gnutls_free(*ciphers);
-			*ciphers = NULL;
+			gnutls_free(ciphers);
+			ciphers = NULL;
 		}
 	}
 
+	*_ciphers = ciphers;
 	gnutls_free(tmp_ciphers);
 	return ret_count;
 }
