@@ -21,8 +21,11 @@
 int gnutls_insertDataBuffer(ContentType type, GNUTLS_STATE state, char *data, int length);
 int gnutls_getDataBufferSize(ContentType type, GNUTLS_STATE state);
 int gnutls_getDataFromBuffer(ContentType type, GNUTLS_STATE state, char *data, int length);
-ssize_t _gnutls_Read(int fd, void *iptr, size_t n, int);
-ssize_t _gnutls_Write(int fd, const void *iptr, size_t n, int );
+ssize_t _gnutls_read_buffered(int fd, GNUTLS_STATE, void *iptr, size_t n, int, ContentType);
+void _gnutls_read_clear_buffer( GNUTLS_STATE);
+int _gnutls_clear_peeked_data( SOCKET cd, GNUTLS_STATE state);
+
+ssize_t _gnutls_write(int fd, const void *iptr, size_t n, int );
 
 /* used in SSL3 */
 int gnutls_getHashDataFromBuffer( GNUTLS_STATE state, char *data, int length);
