@@ -37,7 +37,7 @@
 #define GNUTLS_E_PARSING_ERROR -34
 #define	GNUTLS_E_MPI_PRINT_FAILED -35
 #define GNUTLS_E_AUTH_FAILED -36
-#define GNUTLS_E_GOT_HELLO_REQUEST -37
+#define GNUTLS_E_REHANDSHAKE -37
 #define GNUTLS_E_GOT_APPLICATION_DATA -38
 #define GNUTLS_E_RECORD_LIMIT_REACHED -39
 #define GNUTLS_E_ENCRYPTION_FAILED -40
