@@ -92,6 +92,7 @@ typedef enum AlertDescription { GNUTLS_CLOSE_NOTIFY, GNUTLS_UNEXPECTED_MESSAGE=1
 			} AlertDescription;
 typedef enum CertificateStatus { GNUTLS_CERT_TRUSTED=1, GNUTLS_CERT_NOT_TRUSTED, GNUTLS_CERT_EXPIRED, GNUTLS_CERT_INVALID } CertificateStatus;
 typedef enum CertificateRequest { GNUTLS_CERT_REQUEST=1, GNUTLS_CERT_REQUIRE } CertificateRequest;
+typedef enum CloseRequest { GNUTLS_SHUT_RDWR=0, GNUTLS_SHUT_WR=1 } CloseRequest;
 
 typedef enum HandshakeType { GNUTLS_HELLO_REQUEST, GNUTLS_CLIENT_HELLO, GNUTLS_SERVER_HELLO,
 		     GNUTLS_CERTIFICATE=11, GNUTLS_SERVER_KEY_EXCHANGE,
@@ -204,6 +205,7 @@ typedef struct {
  */
 typedef struct {
 	opaque dnsname[MAX_DNSNAME_SIZE];
+	opaque srp_username[MAX_SRP_USERNAME];
 } TLSExtensions;
 
 /* AUTH_INFO structures MUST NOT contain malloced 
@@ -305,6 +307,11 @@ typedef struct {
 	gnutls_datum			buffer_handshake; /* this is a buffer that holds the current handshake message */
 	ResumableSession		resumable; /* TRUE or FALSE - if we can resume that session */
 	ValidSession			valid_connection; /* true or FALSE - if this session is valid */
+
+	int				may_read; /* if it's 0 then we can read/write, otherwise it's forbiden to read/write
+	                                           */
+	int				may_write;
+
 	AlertDescription		last_alert; /* last alert received */
 	/* this is the compression method we are going to use */
 	CompressionMethod		compression_method;
@@ -322,6 +329,16 @@ typedef struct {
 	/* sockets internals */
 	int				lowat;
 
+					/* this buffer holds a record packet -mostly used for
+					 * non blocking IO.
+					 */
+	opaque				recv_buffer_data[MAX_RECV_SIZE];
+	int				recv_buffer_data_size;
+
+					/* 0 if no peeked data was kept, 1 otherwise.
+					 */
+	int				have_peeked_data;
+
 	/* gdbm */
 	char*				db_name;
 	int				expire_time;
@@ -352,7 +369,6 @@ typedef struct {
 	int				peer_pk_algorithm;
 	/* holds the username got in the srp tls extension
 	 */
-	opaque				srp_username[MAX_SRP_USERNAME];
 } GNUTLS_INTERNALS;
 
 struct GNUTLS_STATE_INT {
