@@ -35,13 +35,6 @@
 #include "gnutls_record.h"
 #include "gnutls_datum.h"
 
-#ifdef HAVE_ERRNO_H
-# include <errno.h>
-#endif
-
-#ifndef EAGAIN
-# define EAGAIN EWOULDBLOCK
-#endif
 
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state) {
 GNUTLS_Version ver;
@@ -61,7 +54,9 @@ void _gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version) {
   * Used to set the lowat value in order for select to check
   * if there are pending data to socket buffer. Used only   
   * if you have changed the default low water value (default is 1).
-  * Normally you will not need that function.
+  * Normally you will not need that function. 
+  * If you plan to use non standard recv() function you should set
+  * this to zero.
   **/
 int gnutls_set_lowat(GNUTLS_STATE state, int num) {
 	state->gnutls_internals.lowat = num;
@@ -339,12 +334,14 @@ svoid *gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_si
   * him of something important (eg. his Certificate could not be verified).
   * If the alert level is Fatal then the peer is expected to close the
   * connection, otherwise he may ignore the alert and continue.
+  * Returns 0 on success.
   *
   **/
 int gnutls_send_alert(SOCKET cd, GNUTLS_STATE state, AlertLevel level, AlertDescription desc)
 {
 	uint8 data[2];
-
+	int ret;
+	
 	memcpy(&data[0], &level, 1);
 	memcpy(&data[1], &desc, 1);
 
@@ -352,37 +349,44 @@ int gnutls_send_alert(SOCKET cd, GNUTLS_STATE state, AlertLevel level, AlertDesc
 	_gnutls_log( "Record: Sending Alert[%d|%d] - %s\n", data[0], data[1], _gnutls_alert2str((int)data[1]));
 #endif
 
-	return gnutls_send_int(cd, state, GNUTLS_ALERT, -1, data, 2, 0);
+	if ( (ret = gnutls_send_int(cd, state, GNUTLS_ALERT, -1, data, 2, 0)) >= 0)
+		return 0;
+	else
+		return ret;
 }
 
 /**
   * gnutls_bye - This function terminates the current TLS/SSL connection.
   * @cd: is a connection descriptor.
   * @state: is a &GNUTLS_STATE structure.
-  * @wait: is an integer
+  * @how: is an integer
   *
-  * Terminates the current TLS/SSL connection. If the return value is 0
-  * you may continue using the TCP connection. The connection should
-  * have been initiated using gnutls_handshake() or similar function.
-  * If 'wait' is non-zero then we will not wait for the other peer to 
-  * close the TLS connection.
+  * Terminates the current TLS/SSL connection. The connection should
+  * have been initiated using gnutls_handshake().
+  * 'how' should be one of GNUTLS_SHUT_RDWR, GNUTLS_SHUT_WR.
   *
-  * This function actually sends the peer a closure alert, and if 'wait'
-  * is zero, will wait for the peer to reply (with a closure alert too).
+  * in case of GNUTLS_SHUT_RDWR then the connection gets terminated and
+  * further receives and sends will be disallowed. If the return
+  * value is zero you may continue using the TCP connection.
+  *
+  * in case of GNUTLS_SHUT_WR then the connection gets terminated and
+  * further sends will be disallowed. In order to reuse the TCP connection
+  * you should wait for an EOF from the peer.
   *
   **/
-int gnutls_bye(SOCKET cd, GNUTLS_STATE state, int wait)
+int gnutls_bye(SOCKET cd, GNUTLS_STATE state, CloseRequest how)
 {
-	int ret;
+	int ret = 0, ret2 = 0;
 
 	ret = gnutls_send_alert(cd, state, GNUTLS_WARNING, GNUTLS_CLOSE_NOTIFY);
 
-	/* receive the closure alert */
-	if (wait==0) gnutls_recv_int(cd, state, GNUTLS_ALERT, -1, NULL, 0, 0); 
-
-	state->gnutls_internals.valid_connection = VALID_FALSE;
-
-	return ret;
+	if ( how == GNUTLS_SHUT_RDWR && ret == 0) {
+		ret2 = gnutls_recv_int(cd, state, GNUTLS_ALERT, -1, NULL, 0, 0); 
+		state->gnutls_internals.may_read = 1;
+	}
+	state->gnutls_internals.may_write = 1;
+	
+	return GMIN(ret, ret2);
 }
 
 /* This function behave exactly like write(). The only difference is 
@@ -403,7 +407,8 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 
 	if (sizeofdata == 0)
 		return 0;
-	if (state->gnutls_internals.valid_connection == VALID_FALSE) {
+
+	if (state->gnutls_internals.valid_connection == VALID_FALSE || state->gnutls_internals.may_write != 0) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
 	}
@@ -446,17 +451,10 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			return cipher_size; /* error */
 		}
 		
-		WRITEuint16( cipher_size, &headers[3]);
-		
-#warning "CHECK if the double write breaks other implementations"
-		if (_gnutls_Write(cd, headers, RECORD_HEADER_SIZE, flags) != RECORD_HEADER_SIZE) {
-			state->gnutls_internals.valid_connection = VALID_FALSE;
-			state->gnutls_internals.resumable = RESUME_FALSE;
-			gnutls_assert();
-			return GNUTLS_E_UNABLE_SEND_DATA;
-		}
+		WRITEuint16( cipher_size-RECORD_HEADER_SIZE, &headers[3]);
+		memcpy( cipher, headers, RECORD_HEADER_SIZE);
 
-		if (_gnutls_Write(cd, cipher, cipher_size, flags) != cipher_size) {
+		if (_gnutls_write(cd, cipher, cipher_size, flags) != cipher_size) {
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
 			gnutls_assert();
@@ -483,12 +481,10 @@ ssize_t gnutls_send_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
 		if (cipher_size<=0) return cipher_size;
 
-		WRITEuint16( cipher_size, &headers[3]);
-
+		WRITEuint16( cipher_size-RECORD_HEADER_SIZE, &headers[3]);
 		memcpy( cipher, headers, RECORD_HEADER_SIZE);
 
-		cipher_size += RECORD_HEADER_SIZE;
-		if (_gnutls_Write(cd, cipher, cipher_size, flags) != cipher_size) {
+		if (_gnutls_write(cd, cipher, cipher_size, flags) != cipher_size) {
 			state->gnutls_internals.valid_connection = VALID_FALSE;
 			state->gnutls_internals.resumable = RESUME_FALSE;
 			gnutls_assert();
@@ -526,17 +522,6 @@ ssize_t _gnutls_send_change_cipher_spec(SOCKET cd, GNUTLS_STATE state)
 
 }
 
-#define RCVLOWAT state->gnutls_internals.lowat /* this is the default for TCP - just don't change that! */
-
-static int _gnutls_clear_peeked_data( SOCKET cd, GNUTLS_STATE state) {
-char peekdata;
-
-	/* this was already read by using MSG_PEEK - so it shouldn't fail */
-	_gnutls_Read( cd, &peekdata, RCVLOWAT, 0); 
-
-	return 0;
-}
-
 #define CHECK_RECORD_VERSION
 
 /* This function behave exactly like read(). The only difference is 
@@ -544,7 +529,7 @@ char peekdata;
  * send (if called by the user the Content is Userdata only)
  * It is intended to receive data, under the current state.
  * flags is the sockets flags to use. Currently only MSG_DONTWAIT is
- * supported.
+ * supported, and should be used together with MSG_WAITALL.
  */
 ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, HandshakeType htype, char *data, size_t sizeofdata, int flags)
 {
@@ -555,35 +540,48 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	ContentType recv_type;
 	uint16 length;
 	uint8 *ciphertext;
-	int ret = 0;
-	int header_size = RECORD_HEADER_SIZE;
+	uint8 *recv_data;
+	int ret;
+	int header_size;
+
+	begin:
+	
+	header_size = RECORD_HEADER_SIZE;
+	ret = 0;
+
+	if (sizeofdata == 0)
+		return 0;
+
+	if (state->gnutls_internals.valid_connection == VALID_FALSE || state->gnutls_internals.may_read!=0) {
+		gnutls_assert();
+		return GNUTLS_E_INVALID_SESSION;
+	}
+	
 	/* If we have enough data in the cache do not bother receiving
 	 * a new packet. (in order to flush the cache)
 	 */
 	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && gnutls_getDataBufferSize(type, state) > 0) {
 		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
 
-		if (type==GNUTLS_APPLICATION_DATA) {
-			/* if the buffer just got empty */
-			if (gnutls_getDataBufferSize(type, state)==0) {
-				_gnutls_clear_peeked_data( cd, state);
-			}
+		/* if the buffer just got empty */
+		if (gnutls_getDataBufferSize(type, state)==0) {
+			_gnutls_clear_peeked_data( cd, state);
 		}
+
 		return ret;
 	}
-
-	if (state->gnutls_internals.valid_connection == VALID_FALSE || sizeofdata==0) {
-		return 0; /* EOF */
-	}
-
+	
 	/* in order for GNUTLS_E_AGAIN to be returned the socket
 	 * must be set to non blocking mode
 	 */
-	if ( (ret = _gnutls_Read(cd, headers, RECORD_HEADER_SIZE, MSG_PEEK|flags)) != RECORD_HEADER_SIZE) {
-		if (ret==(0-EAGAIN)) return GNUTLS_E_AGAIN;
+	if ( (ret = _gnutls_read_buffered(cd, state, headers, RECORD_HEADER_SIZE, flags, -1)) != RECORD_HEADER_SIZE) {
+		if (ret==GNUTLS_E_AGAIN) return ret;
 
 		state->gnutls_internals.valid_connection = VALID_FALSE;
-		if (type==GNUTLS_ALERT) return 0; /* we were expecting close notify */
+		if (type==GNUTLS_ALERT) {
+			gnutls_assert();
+			return 0; /* we were expecting close notify */
+		}
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
@@ -655,15 +653,14 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	ciphertext = gnutls_malloc(length+header_size);
+	recv_data = gnutls_malloc(length+header_size);
+
+	/* check if we have that data into buffer. 
+ 	 */
+	if ( (ret = _gnutls_read_buffered(cd, state, recv_data, header_size+length, flags, recv_type)) != length+header_size) {
+		gnutls_free(recv_data);
+		if (ret==GNUTLS_E_AGAIN) return ret;
 
-/* check if we have that data into buffer. This seems to be
- * expensive - but this is the only way to handle Non Blocking IO.
- */
-	if ( (ret = _gnutls_Read(cd, ciphertext, header_size+length, MSG_PEEK|flags)) != length+header_size) {
-		gnutls_free(ciphertext);
-		
-		if (ret==(0-EAGAIN)) return GNUTLS_E_AGAIN;
 		state->gnutls_internals.valid_connection = VALID_FALSE;
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
@@ -673,42 +670,8 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 /* ok now we are sure that we can read all the data - so
  * move on !
  */
-	if (_gnutls_Read(cd, headers, header_size, 0)!=header_size) {  /* read and clear the headers - again! */
-		gnutls_free(ciphertext);
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		gnutls_assert();
-		return GNUTLS_E_UNKNOWN_ERROR;
-	}
-
-/* Read the whole packet - again? 
- * Here we keep RCVLOWAT bytes in the TCP buffers, only for
- * APPLICATION_DATA data.
- */	
-	if ( type==GNUTLS_APPLICATION_DATA && type==recv_type) {
-		/* get the data - but do not free the buffer in the kernel */
-		ret = _gnutls_Read(cd, ciphertext, length-RCVLOWAT, 0);
-		if (ret>=0)
-			ret += _gnutls_Read(cd, &ciphertext[length-RCVLOWAT], RCVLOWAT, MSG_PEEK);
-
-	} else { /* our - internal data */
-		ret = _gnutls_Read(cd, ciphertext, length, 0);
-	}
-
-	/* Oooops... very rare case since we know that the system HAD 
-	 * received that data.
-	 */
-	if (ret != length) {
-#ifdef RECORD_DEBUG
-		_gnutls_log( "Record: Received packet with length: %d\nExpected %d\n", ret, length);
-#endif
-		gnutls_free(ciphertext);
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		gnutls_assert();
-		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
-	}
-
+	_gnutls_read_clear_buffer( state);
+	ciphertext = &recv_data[header_size];
 	
 	/* decrypt the data we got
 	 */
@@ -728,7 +691,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		state->gnutls_internals.valid_connection = VALID_FALSE;
 		state->gnutls_internals.resumable = RESUME_FALSE;
 		gnutls_assert();
-		gnutls_free(ciphertext);
+		gnutls_free(recv_data);
 		return tmplen;
 	}
 
@@ -739,7 +702,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		_gnutls_log( "Record: ChangeCipherSpec Packet was received\n");
 #endif
 
-		gnutls_free(ciphertext);
+		gnutls_free(recv_data);
 
 		if (tmplen!=sizeofdata) { /* sizeofdata should be 1 */
 			gnutls_assert();
@@ -765,7 +728,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 		return GNUTLS_E_RECORD_LIMIT_REACHED;
 	}
 
-	gnutls_free(ciphertext);
+	gnutls_free(recv_data);
 
 	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
 		gnutls_insertDataBuffer(type, state, (void *) tmpdata, tmplen);
@@ -781,17 +744,15 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			 * the alert is not fatal
 			 */
 			if (tmpdata[1] == GNUTLS_CLOSE_NOTIFY && tmpdata[0] != GNUTLS_FATAL) {
-
 				/* If we have been expecting for an alert do 
 				 * not call close().
 				 */
 				if (type != GNUTLS_ALERT)
-					gnutls_bye(cd, state, 1);
-				
+					gnutls_bye( cd, state, GNUTLS_SHUT_WR);
+
 				gnutls_free(tmpdata);
-				
+
 				return 0; /* EOF */
-/*				return GNUTLS_E_CLOSURE_ALERT_RECEIVED; */
 			} else {
 			
 				/* if the alert is FATAL or WARNING
@@ -817,27 +778,28 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			gnutls_assert();
 	
 			gnutls_free(tmpdata);
-			
+
 			return GNUTLS_E_UNEXPECTED_PACKET;
 		case GNUTLS_APPLICATION_DATA:
-#if 0			
 			/* even if data is unexpected put it into the buffer */
 			gnutls_insertDataBuffer(recv_type, state, (void *) tmpdata, tmplen);
-			/* no peeked data to clear since this packet was unexpected */
-#endif
-			/* We no longer assume this as normal, since
-			 * in this case we don't leave data into kernel
-			 * buffer, thus select() will not return.
-			 * Return an error for now, and we'll handle
-			 * it if there is a need for it.
-			 */
+
 			gnutls_assert();
 			gnutls_free(tmpdata);
-			return GNUTLS_E_UNEXPECTED_PACKET;
-
+			
+			goto begin; /* ok we received the packet, 
+			             * and now we should get the one
+			             * we expected.
+			             */
+			
 			break;
 		case GNUTLS_HANDSHAKE:
-			/* This is only legal if HELLO_REQUEST is received */
+			/* This is only legal if HELLO_REQUEST is received - and we are a client */
+			if (htype!=GNUTLS_HELLO_REQUEST && state->security_parameters.entity==GNUTLS_SERVER) {
+				gnutls_assert();
+				gnutls_free( tmpdata);
+				return GNUTLS_E_UNEXPECTED_PACKET;
+			}
 
 			break;
 		default:
@@ -853,13 +815,12 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 	/* Get Application data from buffer */
 	if ((type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && (recv_type == type)) {
 		ret = gnutls_getDataFromBuffer(type, state, data, sizeofdata);
-		if (type==GNUTLS_APPLICATION_DATA) {
-			/* if the buffer just got empty */
-			if (gnutls_getDataBufferSize(type, state)==0) {
-				_gnutls_clear_peeked_data( cd, state);
-			}
 
+		/* if the buffer just got empty */
+		if (gnutls_getDataBufferSize(type, state)==0) {
+			_gnutls_clear_peeked_data( cd, state);
 		}
+
 		gnutls_free(tmpdata);
 	} else {
 		if (recv_type == GNUTLS_HANDSHAKE) {
@@ -868,7 +829,7 @@ ssize_t gnutls_recv_int(SOCKET cd, GNUTLS_STATE state, ContentType type, Handsha
 			if (ret < 0) {
 				gnutls_assert();
 			} else /* inform the caller */
-				ret = GNUTLS_E_GOT_HELLO_REQUEST;
+				ret = GNUTLS_E_REHANDSHAKE;
 		} else {
 			gnutls_assert();
 			ret = GNUTLS_E_UNEXPECTED_PACKET; 
