@@ -211,9 +211,9 @@ int main(int argc, char** argv)
 	print_info( state);
 
 	printf("- Disconnecting\n");
-	gnutls_bye(sd, state, 0);
+	gnutls_bye(sd, state, GNUTLS_SHUT_RDWR);
 	shutdown( sd, SHUT_WR);
-	close(sd);	
+	close(sd);
 	gnutls_deinit( state);	
 	
 
@@ -288,8 +288,8 @@ int main(int argc, char** argv)
 
 		if (FD_ISSET(sd, &rset)) {
 			bzero(buffer, MAX_BUF+1);
-
 			ret = gnutls_read(sd, state, buffer, MAX_BUF);
+
 			/* remove new line */
 
 			if (gnutls_is_fatal_error(ret) == 1 || ret==0) {
@@ -304,8 +304,8 @@ int main(int argc, char** argv)
 			} else {
 				if (ret==GNUTLS_E_WARNING_ALERT_RECEIVED || ret==GNUTLS_E_FATAL_ALERT_RECEIVED)
 					printf("* Received alert [%d]\n", gnutls_get_last_alert(state));
-				if (ret==GNUTLS_E_GOT_HELLO_REQUEST)
-					printf("* Received HelloRequest message (server asked to rehandshake)\n");
+				if (ret==GNUTLS_E_REHANDSHAKE)
+					printf("* Rehandshake was performed\n");
 
 				if (ret > 0) {
 					printf("- Received[%d]: ", ret);
@@ -320,15 +320,16 @@ int main(int argc, char** argv)
 		if (FD_ISSET(fileno(stdin), &rset)) {
 	
 			if( fgets(buffer, MAX_BUF, stdin) == NULL) {
-				gnutls_bye(sd, state, 0);
+				gnutls_bye(sd, state, GNUTLS_SHUT_WR);
 				user_term = 1;
 				continue;
 			}
 			gnutls_write( sd, state, buffer, strlen(buffer));
 			printf("- Sent: %d bytes\n", strlen(buffer));
+
 		}
 	}
-	if (user_term!=0) gnutls_bye(sd, state, 0);
+	if (user_term!=0) gnutls_bye(sd, state, GNUTLS_SHUT_RDWR);
 	
 	shutdown( sd, SHUT_RDWR); /* no more receptions */
 	close(sd);
