@@ -310,7 +310,7 @@ int read_request(int cd, GNUTLS_STATE state, char *data, int data_size, int rnl)
 			return rc;
 		}
 	}
-fprintf(stderr, "\n");
+
 	*ptr = 0;
 	return n;
 }
@@ -457,7 +457,7 @@ int main(int argc, char **argv)
 			}
 		}
 		printf("\n");
-		gnutls_bye(sd, state, 1); /* do not wait for
+		gnutls_bye(sd, state, GNUTLS_SHUT_WR); /* do not wait for
 		 * the peer to close the connection.
 		 */
 		close(sd);
